package com.bcxin.tenant.open.dubbo.reader.providers.impls;
import com.bcxin.tenant.open.document.domains.documents.*;
import com.bcxin.tenant.open.document.domains.repositories.RdEmployeeDocumentRepository;
import com.bcxin.tenant.open.dubbo.reader.providers.transfers.DataTransfer;
import com.bcxin.tenant.open.infrastructure.redis.RedisConfig;
import com.bcxin.tenant.open.infrastructures.EntityCollection;
import com.bcxin.tenant.open.infrastructures.TenantContext;
import com.bcxin.tenant.open.infrastructures.TenantEmployeeContext;
import com.bcxin.tenant.open.infrastructures.enums.*;
import com.bcxin.tenant.open.infrastructures.exceptions.ArgumentTenantException;
import com.bcxin.tenant.open.infrastructures.exceptions.BadTenantException;
import com.bcxin.tenant.open.infrastructures.exceptions.UnAuthorizedTenantException;
import com.bcxin.tenant.open.infrastructures.utils.BusinessUtil;
import com.bcxin.tenant.open.infrastructures.utils.StringUtil;
import com.bcxin.tenant.open.jdks.EmployeeReaderRpcProvider;
import com.bcxin.tenant.open.infrastructures.constants.BusinessConstants;
import com.bcxin.tenant.open.jdks.requests.EmployeeAdvanceRequest;
import com.bcxin.tenant.open.jdks.requests.EmployeeDutyStatusRequest;
import com.bcxin.tenant.open.jdks.requests.EmployeeSearchRequest;
import com.bcxin.tenant.open.jdks.requests.enums.EmployeeResourceType;
import com.bcxin.tenant.open.jdks.responses.EmployeeAdvanceResponse;
import com.bcxin.tenant.open.jdks.responses.EmployeeDetailResponse;
import com.bcxin.tenant.open.jdks.responses.EmployeeOverviewResponse;
import com.bcxin.tenant.open.jdks.utils.RequestUtil;
import com.bcxin.tenant.open.jdks.responses.EmployeeDutyStatusResponse;
import com.bcxin.tenant.open.jdks.responses.EmployeeCountResponse;
import com.redis.om.spring.search.stream.EntityStream;
import com.redis.om.spring.search.stream.SearchStream;
import org.apache.dubbo.config.annotation.DubboService;
import org.springframework.data.geo.Distance;
import org.springframework.data.geo.Point;
import org.springframework.data.redis.domain.geo.Metrics;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;
import redis.clients.jedis.search.aggr.SortedField;

import java.util.*;
import java.util.stream.Collectors;

@DubboService
public class EmployeeReaderRpcProviderImpl implements EmployeeReaderRpcProvider {
    private final DataTransfer dataTransfer;
    private final RdEmployeeDocumentRepository employeeDocumentRepository;
    private final EntityStream entityStream;

    public EmployeeReaderRpcProviderImpl(DataTransfer dataTransfer,
                                         RdEmployeeDocumentRepository employeeDocumentRepository,
                                         EntityStream entityStream) {
        this.dataTransfer = dataTransfer;
        this.employeeDocumentRepository = employeeDocumentRepository;
        this.entityStream = entityStream;
    }

    @Override
    public EmployeeDetailResponse get(String id, DeskType deskType) {
        Optional<RdEmployeeDocument> documentOptional =
                employeeDocumentRepository.findById(id);

        if (!documentOptional.isPresent()) {
            return null;
        }

        return this.dataTransfer.translate(deskType, documentOptional.get(),entityStream);
    }

    @Override
    public Collection<EmployeeOverviewResponse> search(EmployeeSearchRequest request) {
        if (CollectionUtils.isEmpty(request.getDeskTypes())) {
            throw new ArgumentTenantException("参数异常; 工作台类型(deskTypes)不能为空");
        }

        int pageIndex = request.getPageIndex();
        if (pageIndex < 1) {
            pageIndex = 1;
        }
        TenantEmployeeContext.TenantUserModel userModel =
                TenantContext.getInstance().getUserContext().get();
        if (userModel == null) {
            throw new UnAuthorizedTenantException();
        }

        /**
         * 临保指挥调度按照安保资源的方式进行过滤
         */
        EmployeeSearchRequest.SearchModel searchModel = request.getSearchModel();

        /**
         * 根据监管归属进行过滤, 暂时不支持企业级的调度台过滤
         */
        SearchStream<RdEmployeeDocument> employeeDocumentSearchStream =
                this.entityStream.of(RdEmployeeDocument.class);
        /**
         * 非考场的查询和非根据驻勤点Id查询的时候, 不过滤其他
         */
        if (searchModel != EmployeeSearchRequest.SearchModel.ExamSiteRoom &&
                searchModel!= EmployeeSearchRequest.SearchModel.SecurityStation) {
            Set<String> selectedResources
                    = ResourceType.getResources(request.getDeskTypes());
            if (!CollectionUtils.isEmpty(selectedResources)) {
                employeeDocumentSearchStream =
                        employeeDocumentSearchStream.filter(RdEmployeeDocument$.RESOURCE_TYPES.in(
                                selectedResources.toArray(ix -> new String[ix])
                        ));
            }

            /**
             * 注意: 这边是采用and的条件的方式进行查询的
             */
            if (!CollectionUtils.isEmpty(request.getResourceTypes())) {
                Collection<ResourceType> excludedResourceTypes =
                        RequestUtil.getExcludedResourceTypes(request.getResourceTypes());
                Collection<ResourceType> includedResourceTypes =
                        RequestUtil.getResourceTypes(request.getResourceTypes());

                for (ResourceType rn : includedResourceTypes) {
                    employeeDocumentSearchStream =
                            employeeDocumentSearchStream.filter(
                                    RdEmployeeDocument$.RESOURCE_TYPES.in(rn.name())
                            );
                }

                /**
                 * 排除如下的选项
                 */
                if (!CollectionUtils.isEmpty(excludedResourceTypes)) {
                    employeeDocumentSearchStream = employeeDocumentSearchStream.filter(String.format("-@resourceTypes:{%s}",
                            excludedResourceTypes.stream().map(ii -> ii.name()).collect(Collectors.joining(" | "))
                    ));
                }
            }

            RoleType roleType = RoleType.create(userModel.getOrgInstitutional(), userModel.getOrgName());
            if (userModel.isSuperviseDepartRole()) {
                String[] superviseDepartIds =
                        userModel.getMatchedSuperviseDepartIds(request.getSuperviseDepartIds())
                                .toArray(ii -> new String[ii]);
                if (superviseDepartIds.length == 0) {
                    return Collections.EMPTY_LIST;
                }

                employeeDocumentSearchStream = employeeDocumentSearchStream
                        .filter(RdEmployeeDocument$.SCOPE_PERMISSIONS.in(superviseDepartIds));

            } else {
                /**
                 * 支持本企业或者内保调度台调度
                 */
                employeeDocumentSearchStream = employeeDocumentSearchStream
                        .filter(RdEmployeeDocument$.SCOPE_PERMISSIONS.in(userModel.getNoSuperviseScopePermissions()));
                if (roleType == RoleType.Company) {
                    String[] excludeValues = new String[]{
                            BusinessConstants.DefaultEmptyValue
                    };

                    employeeDocumentSearchStream = employeeDocumentSearchStream
                            .filter(RdEmployeeDocument$.SECURITY_STATION_ID.notEq(excludeValues));
                }
            }

            employeeDocumentSearchStream = filterByLimitedStations(userModel, roleType, employeeDocumentSearchStream);

            if (StringUtils.hasLength(request.getProjectId())) {
                String[] projIds = new String[]{request.getProjectId()};

                employeeDocumentSearchStream = employeeDocumentSearchStream.filter(RdEmployeeDocument$.SCOPE_PERMISSIONS.in(projIds));
            }
        }

        /**
         * 在马拉松中; 此stationType作为活动承办方的人员类型（志愿者、裁判、医护人员、反制人员）进行过滤.
         */
        if (!CollectionUtils.isEmpty(request.getStationTypes())) {
            String[] stationTypes = request.getStationTypes()
                    .toArray(new String[request.getStationTypes().size()]);

            employeeDocumentSearchStream
                    = employeeDocumentSearchStream
                    .filter(RdEmployeeDocument$.STATION_TYPES.in(stationTypes));
        }

        boolean skipPage = request.isSkipPageLimit();
        switch (searchModel) {
            case Geo -> {
                employeeDocumentSearchStream =
                        employeeDocumentSearchStream
                                .filter(RdEmployeeDocument$.LON_LAT.near(
                                        new Point(request.getLongitude(), request.getLatitude()),
                                        new Distance(request.getDistance(), Metrics.METERS)
                                ));

                skipPage = true;
            }
            case Keyword -> {
                if (!StringUtils.hasLength(request.getKeyword())) {
                    throw new BadTenantException("搜索关键字不可以为空");
                }

                employeeDocumentSearchStream =
                        filterByKeyword(employeeDocumentSearchStream, request.getKeyword());
            }

            case Other -> {
                Set<Boolean> dispatchables = new HashSet<>();
                if (request.getDispatchable().isPresent()) {
                    dispatchables.add(request.getDispatchable().get());
                } else {
                    dispatchables.add(true);
                    dispatchables.add(false);
                }

                if (request.getDispatchable().isPresent()) {
                    employeeDocumentSearchStream =
                            employeeDocumentSearchStream.filter(RdEmployeeDocument$.DISPATCHABLE.eq(request.getDispatchable().get()));
                }
                skipPage = true;
            }
            case SecurityStation -> {
                if (CollectionUtils.isEmpty(request.getSecurityStationIds())) {
                    return Collections.EMPTY_LIST;
                }

                if (!CollectionUtils.isEmpty(request.getSecurityStationIds())) {
                    String[] stationIds = request.getSecurityStationIds().toArray(ix -> new String[ix]);
                    employeeDocumentSearchStream =
                            employeeDocumentSearchStream
                                    .filter(RdEmployeeDocument$.SECURITY_STATION_ID.in(stationIds));
                }

                employeeDocumentSearchStream =
                        filterByKeyword(employeeDocumentSearchStream, request.getKeyword());

                employeeDocumentSearchStream = employeeDocumentSearchStream.sorted(RdEmployeeDocument$.DISPATCHABLE, SortedField.SortOrder.DESC);
            }
            /**
             * 调度通讯的人员功能
             */
            case DispatchBookAddress -> {
                if (CollectionUtils.isEmpty(request.getSecurityStationIds())) {
                    return Collections.EMPTY_LIST;
                }

                employeeDocumentSearchStream =
                        employeeDocumentSearchStream
                                .filter(RdEmployeeDocument$.SECURITY_STATION_ID.in(request.getSecurityStationIds().toArray(ix -> new String[ix])));

                employeeDocumentSearchStream =
                        filterByKeyword(employeeDocumentSearchStream, request.getKeyword());

                employeeDocumentSearchStream = employeeDocumentSearchStream.sorted(RdEmployeeDocument$.DISPATCHABLE, SortedField.SortOrder.DESC);
            }
            case SelectedEmployees -> {
                if (CollectionUtils.isEmpty(request.getIds())) {
                    return Collections.EMPTY_LIST;
                }

                String[] selectedIds = request.getIds().stream().filter(ii -> !StringUtil.isEmpty(ii))
                        .distinct().toArray(ix -> new String[ix]);
                if (selectedIds == null || selectedIds.length == 0) {
                    return Collections.EMPTY_LIST;
                }

                employeeDocumentSearchStream =
                        employeeDocumentSearchStream
                                .filter(RdEmployeeDocument$.ID.in(selectedIds));
            }
            case ExamSiteRoom -> {
                if (StringUtil.isEmpty(request.getExamSiteRoomId())) {
                    return Collections.EMPTY_LIST;
                }

                SearchStream<RdSystemExamInfoDocument> examInfoDocumentSearchStream
                        = this.entityStream.of(RdSystemExamInfoDocument.class)
                        .filter(RdSystemExamInfoDocument$.EXAM_SITE_ROOM_ID.eq(request.getExamSiteRoomId()));
                Collection<RdSystemExamInfoDocument> selectedTeacherMultipleIds =
                        examInfoDocumentSearchStream.collect(Collectors.toList());
                if (CollectionUtils.isEmpty(selectedTeacherMultipleIds)) {
                    return Collections.EMPTY_LIST;
                }

                Collection<String> teacherIds =
                        selectedTeacherMultipleIds.stream()
                                .map(ii -> ii.getTeacherIds())
                                .flatMap(ii -> Arrays.stream(ii.split(";")))
                                .filter(ii -> StringUtils.hasLength(ii))
                                .distinct()
                                .collect(Collectors.toList());

                employeeDocumentSearchStream = employeeDocumentSearchStream.filter(RdEmployeeDocument$.ID.in(teacherIds.toArray(ii -> new String[ii])));
                skipPage = true;
            }
            case SecurityResource -> {
                /**
                 * 辖区在上面已经进行过滤
                 */
                skipPage = true;
                if (!userModel.isSuperviseDepartRole() && !CollectionUtils.isEmpty(request.getSuperviseDepartIds())) {
                    employeeDocumentSearchStream = employeeDocumentSearchStream
                            .filter(RdEmployeeDocument$.SCOPE_PERMISSIONS.in(request.getSuperviseDepartIds().toArray(ii -> new String[ii])));
                }

                if (request.getFollowed() != null) {
                    if (request.getFollowed()) {
                        employeeDocumentSearchStream =
                                employeeDocumentSearchStream.filter(
                                        RdEmployeeDocument$.FOLLOWED_DEVICE_NOS.in(userModel.getTencentUserId())
                                );
                    } else {
                        /**
                         * 关注人不在列表中
                         */
                        Set<String> noDeviceNos = new HashSet<>();
                        noDeviceNos.add(userModel.getTencentUserId());

                        employeeDocumentSearchStream =
                                employeeDocumentSearchStream.filter(
                                        RdEmployeeDocument$.FOLLOWED_DEVICE_NOS.containsNone(noDeviceNos)
                                );
                    }
                }
                employeeDocumentSearchStream =
                        filterByKeyword(employeeDocumentSearchStream, request.getKeyword());
            }
        }

        if (request.getOnDuty() != null) {
            if (request.getOnDuty()) {
                employeeDocumentSearchStream =
                        employeeDocumentSearchStream.filter(RdEmployeeDocument$.DISPATCHABLE.eq(true));
            } else {
                String[] dutyEqs = new String[]{"true"};
                employeeDocumentSearchStream =
                        employeeDocumentSearchStream.filter(RdEmployeeDocument$.DISPATCHABLE.notEq(dutyEqs));
            }
        }

        /**
         * 针对赛演进行限制;
         * 针对管理单位，如果有限制的话, 则进行限制
         * 针对所负责的驻勤点相关人员进行过滤
         * todo: 针对承办单位的情况(不应该限制)
         *
         * 根据驻勤Id的时候就不过滤其他了.
         */
        if (searchModel != EmployeeSearchRequest.SearchModel.SecurityStation) {
            employeeDocumentSearchStream =
                    filterByUserResource(userModel, employeeDocumentSearchStream, request.getResponsibleOfStationIds());
        }

        Collection<RdEmployeeDocument> employeeDocuments = null;
        if (skipPage) {
            if (searchModel != EmployeeSearchRequest.SearchModel.SecurityResource) {
                employeeDocuments = employeeDocumentSearchStream
                        /**
                         * 这个一定要再map之前, 否则无法执行
                         */
                        .limit(RedisConfig.MAX_LIMIT)
                        .collect(Collectors.toList());
            } else {
                employeeDocuments = employeeDocumentSearchStream
                        .limit(RedisConfig.MAX_LIMIT)
                        .collect(Collectors.toList());
            }
        } else {
            employeeDocuments = employeeDocumentSearchStream
                    .skip((pageIndex - 1) * request.getPageSize())
                    .limit(request.getPageSize())
                    .collect(Collectors.toList());
        }

        return this.dataTransfer.translate(request.getDeskTypes(), searchModel, employeeDocuments, entityStream);
    }

    @Override
    public Collection<String> getEmployeeIdsByOrganizationIds(Collection<String> organizationIds) {
        if (CollectionUtils.isEmpty(organizationIds)) {
            return Collections.emptyList();
        }

        String[] organizationIdsArray = organizationIds.toArray(new String[organizationIds.size()]);

        Collection<String> ids =
                this.entityStream.of(RdEmployeeDocument.class)
                        .filter(RdEmployeeDocument$.ORGANIZATION_ID.in(organizationIdsArray))
                        .collect(Collectors.toList()).stream()
                        .map(ix -> ix.getId()).distinct()
                        .collect(Collectors.toList());

        return ids;
    }

    @Override
    public Collection<String> getEmployeeIdsByStationIds(Collection<String> stationIds) {
        if (CollectionUtils.isEmpty(stationIds)) {
            return Collections.emptyList();
        }

        String[] stationIdsArray = stationIds.toArray(new String[stationIds.size()]);

        Collection<String> ids =
                this.entityStream.of(RdEmployeeDocument.class)
                        .filter(RdEmployeeDocument$.SECURITY_STATION_ID.in(stationIdsArray))
                        .collect(Collectors.toList()).stream()
                        .map(ix -> ix.getId()).collect(Collectors.toList());

        return ids;
    }

    @Override
    public Collection<EmployeeOverviewResponse> getEmployeeOverviewByIds(Collection<String> ids) {
        if (CollectionUtils.isEmpty(ids)) {
            return Collections.EMPTY_LIST;
        }
        SearchStream<RdEmployeeDocument> employeeDocumentSearchStream =
                this.entityStream.of(RdEmployeeDocument.class);
        employeeDocumentSearchStream =
                employeeDocumentSearchStream.filter(RdEmployeeDocument$.ID.in(ids.toArray(new String[ids.size()])));

        Collection<RdEmployeeDocument> documents = employeeDocumentSearchStream.collect(Collectors.toList());

        if (CollectionUtils.isEmpty(documents)) {
            return Collections.EMPTY_LIST;
        }

        return this.dataTransfer.translate(Collections.singleton(DeskType.Normal), EmployeeSearchRequest.SearchModel.Other, documents, entityStream);
    }

    @Override
    public EntityCollection<EmployeeAdvanceResponse> search(EmployeeAdvanceRequest request) {
        TenantEmployeeContext.TenantUserModel userModel = TenantContext.getInstance().getUserContext().get();
        if (userModel == null) {
            throw new UnAuthorizedTenantException();
        }

        SearchStream<RdEmployeeDocument> searchStream =
                this.entityStream.of(RdEmployeeDocument.class);

        /**
         * 过滤调度台类型
         */
        Set<String> selectedResources
                = ResourceType.getResources(request.getDeskTypes());
        if (!CollectionUtils.isEmpty(selectedResources)) {
            searchStream =
                    searchStream.filter(RdEmployeeDocument$.RESOURCE_TYPES.in(
                            selectedResources.toArray(ix -> new String[ix])
                    ));
        }

        searchStream =
                searchStream
                        .filter(RdEmployeeDocument$.DUTY_STATUS.in(DutySignInType.None.name()));

        String[] excludeValues = new String[]{
                BusinessConstants.DefaultEmptyValue
        };
        searchStream = searchStream.filter(RdEmployeeDocument$.SECURITY_STATION_ID.notEq(excludeValues));
        if (userModel.isSuperviseDepartRole()) {
            String[] superviseDepartIds
                    = userModel.getAssignedSuperviseDepartIds();
            if (superviseDepartIds == null || superviseDepartIds.length == 0) {
                return EntityCollection.create(Collections.EMPTY_LIST, request.getPageSize(), 0l);
            }

            if (request.getSuperviseDepartIds() != null && request.getSuperviseDepartIds().size() > 0) {
                superviseDepartIds = Arrays.stream(superviseDepartIds)
                        .filter(ii -> request.getSuperviseDepartIds().contains(ii))
                        .toArray(sz -> new String[sz]);
            }

            if (superviseDepartIds.length == 0) {
                return EntityCollection.create(Collections.EMPTY_LIST, request.getPageSize(), 0l);
            }

            searchStream = searchStream.filter(RdEmployeeDocument$.SCOPE_PERMISSIONS.in(superviseDepartIds));
            /**
             * 针对监管的话, 排除本身的数据和组织的数据
             */
            Collection<String> allExcludedOrgIds = new HashSet<>();
            allExcludedOrgIds.addAll(Arrays.stream(superviseDepartIds).toList());
            allExcludedOrgIds.add(userModel.getOrganizationId());

            String[] orgIds = allExcludedOrgIds.toArray(size -> new String[size]);
            searchStream = searchStream.filter(RdEmployeeDocument$.ORGANIZATION_ID.notEq(orgIds));
        } else {
            searchStream = searchStream.filter(RdEmployeeDocument$.SCOPE_PERMISSIONS.in(userModel.getNoSuperviseScopePermissions()));

            if (request.getSuperviseDepartIds() != null && request.getSuperviseDepartIds().size() > 0) {
                String[] superviseDepartIds = request.getSuperviseDepartIds().toArray(sz -> new String[sz]);
                searchStream =
                        searchStream.filter(RdEmployeeDocument$.SCOPE_PERMISSIONS.in(superviseDepartIds));
            }
        }

        if (StringUtils.hasLength(request.getCompanyName())) {
            searchStream = searchStream.filter(RdEmployeeDocument$.COMPANY_NAME.startsWith(request.getCompanyName()));
        }

        if (StringUtils.hasLength(request.getName())) {
            searchStream = searchStream.filter(RdEmployeeDocument$.NAME.startsWith(request.getName()));
        }

        if (StringUtils.hasLength(request.getSecurityStationName())) {
            searchStream = searchStream.filter(RdEmployeeDocument$.SECURITY_STATION_NAME.startsWith(request.getSecurityStationName()));
        }

        /**
         * 所属临保项目
         */
        if (StringUtils.hasLength(request.getProjectId())) {
            searchStream =
                    searchStream.filter(RdEmployeeDocument$.SCOPE_PERMISSIONS.in(request.getProjectId()));
        }

        if (StringUtils.hasLength(request.getIdCardNo())) {
            searchStream = searchStream.filter(RdEmployeeDocument$.ID_CARD_NO.startsWith(request.getIdCardNo()));
        }

        /**
         * 针对赛演进行限制;
         * 针对管理单位，如果有限制的话, 则进行限制
         * 针对所负责的驻勤点相关人员进行过滤
         * todo: 针对承办单位的情况(不应该限制)
         */
        searchStream =
                filterByUserResource(userModel,searchStream,Collections.EMPTY_LIST);

        long totalCount = searchStream.count();
        int pageIndex = request.getPageIndex();
        if (pageIndex < 1) {
            pageIndex = 1;
        }

        Collection<EmployeeAdvanceResponse>
                data = searchStream.skip((pageIndex - 1) * request.getPageSize()).limit(request.getPageSize())
                .collect(Collectors.toList())
                .stream().map(ii -> EmployeeAdvanceResponse.create(
                        ii.getId(),
                        ii.getName(),
                        ii.getSex(),
                        ii.getIdCardNo(),
                        ii.getContact(),
                        ii.getOrganizationId(),
                        ii.getCompanyName(),
                        ii.getSuperviseDepartId(),
                        ii.getSuperviseDepartName(),
                        ii.getAuthenticatedStatus(),
                        StringUtils.hasLength(ii.getSecurityCertificateNo()),
                        "#1".equals(ii.getSecurityStationId()) ? "" : ii.getSecurityStationId(),
                        "#1".equals(ii.getSecurityStationName()) ? "" : ii.getSecurityStationName()
                )).collect(Collectors.toList());

        return EntityCollection.create(data, request.getPageSize(), totalCount);
    }

    @Override
    public EmployeeCountResponse getEmployeeNumberByStationId(String stationId) {
        TenantEmployeeContext.TenantUserModel userModel = TenantContext.getInstance().getUserContext().get();
        if (userModel == null) {
            throw new UnAuthorizedTenantException();
        }

        Collection<RdEmployeeDocument> employeeDocuments = this.entityStream.of(RdEmployeeDocument.class)
                        .filter(RdEmployeeDocument$.SECURITY_STATION_ID.eq(stationId)).collect(Collectors.toList());
        Integer totalNumber = employeeDocuments.size();
        Integer ondutyNumber = employeeDocuments.stream()
                .filter(ii -> ii.getDutyStatus().equals(DutySignInType.SignIn.name()))
                .collect(Collectors.toList()).size();

        return EmployeeCountResponse.create(totalNumber,ondutyNumber,totalNumber - ondutyNumber);
    }

    @Override
    public Collection<EmployeeDutyStatusResponse> getEmployeesByStationIdAndDutyStatus(EmployeeDutyStatusRequest request) {
        TenantEmployeeContext.TenantUserModel userModel = TenantContext.getInstance().getUserContext().get();
        if (userModel == null) {
            throw new UnAuthorizedTenantException();
        }

        String[] dutyStatusArr = request.isOnDuty() ? new String[]{DutySignInType.SignIn.name()}
                            : new String[]{DutySignInType.SignOut.name(), DutySignInType.None.name()};
        int pageIndex = request.getPageIndex();
        if(pageIndex < 1){
            pageIndex = 1;
        }

        Collection<EmployeeDutyStatusResponse> responses = this.entityStream.of(RdEmployeeDocument.class)
                .filter(RdEmployeeDocument$.SECURITY_STATION_ID.eq(request.getStationId()))
                .filter(RdEmployeeDocument$.DUTY_STATUS.in(dutyStatusArr))
                .skip((pageIndex-1)*request.getPageSize())
                .limit(request.getPageSize())
                .collect(Collectors.toList())
                .stream().map(ii -> EmployeeDutyStatusResponse.create(
                        ii.getTenantEmployeeId(),
                        ii.getTenantUserId(),
                        ii.getIdCardNo(),
                        ii.getOrganizationId(),
                        ii.getName(),
                        ii.getContact()
                )).collect(Collectors.toList());
        return responses;
    }

    private Collection<String> getFollowedFilterEmployeeIds(TenantEmployeeContext.TenantUserModel userModel, boolean followed) {
        String[] superviseDepartIds = userModel.getAssignedSuperviseDepartIds();
        if (superviseDepartIds == null) {
            return Collections.EMPTY_LIST;
        }

        SearchStream<TenantContentFollowDocument> documentSearchStream = this.entityStream.of(TenantContentFollowDocument.class);
        documentSearchStream.filter(TenantContentFollowDocument$.FOLLOWER_ID.in(userModel.getSuperviseDepartId()));

        return Collections.EMPTY_LIST;
    }

    private static SearchStream<RdEmployeeDocument> filterByKeyword(SearchStream<RdEmployeeDocument> employeeDocumentSearchStream,
                                                                 String keyword) {
        /**
         * changed by lhc: 20231129; 由于人口的需求, 派出所可以根据关键字和资源类型进行过滤
         * 前端会根据当前的账号类型来判断是否为派出所, 如果是派出所才会调用这个API
         * 驻勤点也类似
         */
        if (!StringUtil.isEmpty(keyword)) {
            if (StringUtil.isNumber(keyword)) {
                employeeDocumentSearchStream =
                        employeeDocumentSearchStream
                                .filter(RdEmployeeDocument$.ID_CARD_NO.eq(keyword.trim()));
            } else {
                employeeDocumentSearchStream =
                        employeeDocumentSearchStream
                                .filter(RdEmployeeDocument$.NAME.eq(keyword.trim()));
            }
        }

        return employeeDocumentSearchStream;
    }

    private static SearchStream<RdEmployeeDocument> filterByLimitedStations(TenantEmployeeContext.TenantUserModel userModel,
                                                                         RoleType roleType,
                                                                         SearchStream<RdEmployeeDocument> employeeDocumentSearchStream
                                                                         ) {

        /**
         * 内保单位的话; 则除了主管理员之外, 仅指定的保卫管理员可以查看到数据
         */
        if(roleType==RoleType.Proprietor) {
            if (userModel.getPermission() != null && userModel.getPermission().isDomainAdmin()) {
                return employeeDocumentSearchStream;
            }

            String[] limitedPermissionStationIds = userModel.getLimitedPermissionStations();
            employeeDocumentSearchStream = employeeDocumentSearchStream
                    .filter(RdEmployeeDocument$.SECURITY_STATION_ID.in(limitedPermissionStationIds));

            return employeeDocumentSearchStream;
        }

        if (userModel.getAccountType() == DispatchAccountType.AppDesk) {

            /**
             * 企业/群防群治和派出所将限制
             * 社区警务的负责人仅查看到对应社区警务的人员信息
             * 驻勤点负责人仅查看对应的驻勤点的人员信息
             * */
            if (roleType == RoleType.Company ||
                    roleType == RoleType.PoliceStation ||
                    roleType == RoleType.PopulationPoliceStation) {
                String[] limitedPermissionStationIds = userModel.getLimitedPermissionStations();
                /**
                 * 针对企业及群防群治企业
                 */
                if (limitedPermissionStationIds != null &&
                        limitedPermissionStationIds.length > 0) {
                    if (roleType == RoleType.Company) {
                        employeeDocumentSearchStream = employeeDocumentSearchStream
                                .filter(RdEmployeeDocument$.SECURITY_STATION_ID.in(userModel.getLimitedPermissionStations()));
                    } else {
                        String filteredValues = Arrays.stream(limitedPermissionStationIds)
                                .collect(Collectors.joining(" | "));
                        employeeDocumentSearchStream = employeeDocumentSearchStream
                                .filter(String.format("(@resourceTypes:{Population} @securityStationId:{%s}) | -(@resourceTypes:{Population})", filteredValues));
                    }
                }
            }

        }

        return employeeDocumentSearchStream;
    }

    /**
     * 针对赛演进行限制;
     * 针对管理单位，如果有限制的话, 则进行限制
     * 针对所负责的驻勤点相关人员进行过滤
     * todo: 针对承办单位的情况(不应该限制)
     */
    private static SearchStream<RdEmployeeDocument> filterByUserResource(
            TenantEmployeeContext.TenantUserModel userModel,
            SearchStream<RdEmployeeDocument> employeeDocumentSearchStream,
            /**
             * 根据所负责的驻勤点进行过滤; 在业务管理的未激活人数无需传入
             */
            Collection<String> responsibleOfStationIds
            ) {
        /**
         * 针对赛演进行限制;
         * 针对管理单位，如果有限制的话, 则进行限制
         * todo: 针对承办单位的情况(不应该限制)
         */
        if (BusinessUtil.isEventOrganizer(userModel.getOrgInstitutional())) {
            Collection<String> limitedPermissionIds = new HashSet<>();
            if (!CollectionUtils.isEmpty(responsibleOfStationIds)) {
                limitedPermissionIds.addAll(responsibleOfStationIds);
            }
            limitedPermissionIds.addAll(Arrays.stream(userModel.getLimitedPermissionStations()).toList());

            Set<String> limitedValidPermissionStationIds = limitedPermissionIds.stream()
                    .filter(ii -> !BusinessConstants.DefaultEmptyValue.equalsIgnoreCase(ii))
                    .collect(Collectors.toSet());

            if (!CollectionUtils.isEmpty(limitedValidPermissionStationIds)) {
                String[] limitedPermissionStationIds = limitedValidPermissionStationIds.toArray(ix -> new String[ix]);
                employeeDocumentSearchStream = employeeDocumentSearchStream
                        .filter(RdSecurityStationDocument$.SCOPE_PERMISSIONS.in(limitedPermissionStationIds));
            }
        } else {
            /**
             * 所负责的项目
             */
            if (!CollectionUtils.isEmpty(responsibleOfStationIds)) {
                String[] responseStationIds = responsibleOfStationIds.toArray(ix -> new String[ix]);
                employeeDocumentSearchStream =
                        employeeDocumentSearchStream
                                .filter(RdEmployeeDocument$.SCOPE_PERMISSIONS.in(responseStationIds));
            }
        }

        return employeeDocumentSearchStream;
    }
}