package com.bcxin.tenant.open.dubbo.reader.providers.impls;

import com.bcxin.tenant.open.document.domains.documents.*;
import com.bcxin.tenant.open.dubbo.reader.providers.utils.TermUtil;
import com.bcxin.tenant.open.infrastructures.TenantContext;
import com.bcxin.tenant.open.infrastructures.TenantEmployeeContext;
import com.bcxin.tenant.open.infrastructures.enums.ExamType;
import com.bcxin.tenant.open.infrastructures.exceptions.BadTenantException;
import com.bcxin.tenant.open.infrastructures.exceptions.NoFoundTenantException;
import com.bcxin.tenant.open.infrastructures.exceptions.UnAuthorizedTenantException;
import com.bcxin.tenant.open.infrastructures.utils.DateUtils;
import com.bcxin.tenant.open.jdks.ExamSiteRoomReaderRpcProvider;
import com.bcxin.tenant.open.jdks.requests.BroadcastPoliceIndicentMessageContentItem;
import com.bcxin.tenant.open.jdks.requests.ExamSiteRoomSearchRequest;
import com.bcxin.tenant.open.jdks.responses.ExamSiteRoomDetailResponse;
import com.bcxin.tenant.open.jdks.responses.ExamSiteRoomOverviewResponse;
import com.redis.om.spring.search.stream.EntityStream;
import com.redis.om.spring.search.stream.SearchStream;
import org.apache.dubbo.config.annotation.DubboService;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.sql.Timestamp;
import java.time.Instant;
import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;

@DubboService
public class ExamSiteRoomReaderRpcProviderImpl implements ExamSiteRoomReaderRpcProvider {
    private final EntityStream entityStream;

    public ExamSiteRoomReaderRpcProviderImpl(EntityStream entityStream) {
        this.entityStream = entityStream;
    }

    @Override
    public Collection<ExamSiteRoomOverviewResponse> search(ExamSiteRoomSearchRequest request) {
        TenantEmployeeContext.TenantUserModel userModel = TenantContext.getInstance().getUserContext().get();
        if (userModel == null) {
            throw new UnAuthorizedTenantException();
        }

        SearchStream<RdExamSiteRoomDocument> examSiteRoomDocumentStream =
                this.entityStream.of(RdExamSiteRoomDocument.class);

        SearchStream<RdSystemExamInfoDocument> systemExamInfoDocumentSearchStream =
                this.entityStream.of(RdSystemExamInfoDocument.class);
        if (userModel.isSuperviseDepartRole()) {
            String[] superviseDepartIds = userModel.getAssignedSuperviseDepartIds();
            /**
             * 根据监管归属进行过滤, 暂时不支持企业级的调度台过滤
             */

            if (request.getSearchModel() == ExamSiteRoomSearchRequest.SearchModel.SecurityResource &&
                    !CollectionUtils.isEmpty(request.getSuperviseDepartIds())) {
                superviseDepartIds = Arrays.stream(superviseDepartIds)
                        .filter(ix -> request.getSuperviseDepartIds().contains(ix))
                        .toArray(size -> new String[size]);
            }

            if (superviseDepartIds.length == 0) {
                return Collections.EMPTY_LIST;
            }

            examSiteRoomDocumentStream = examSiteRoomDocumentStream
                    .filter(RdExamSiteRoomDocument$.SUPERVISE_DEPART_ID.in(superviseDepartIds));

            systemExamInfoDocumentSearchStream = systemExamInfoDocumentSearchStream
                    .filter(RdSystemExamInfoDocument$.SUPERVISE_DEPART_ID.in(superviseDepartIds));
        } else {
            /**
             * 支持本企业调度台调度
             */
            examSiteRoomDocumentStream = examSiteRoomDocumentStream
                    .filter(RdEmployeeDocument$.ORGANIZATION_ID.eq(userModel.getOrganizationId()));

            systemExamInfoDocumentSearchStream = systemExamInfoDocumentSearchStream
                    .filter(RdSystemExamInfoDocument$.ORGANIZATION_ID.eq(userModel.getOrganizationId()));
        }

        switch (request.getSearchModel()) {
            case SecurityResource -> {
                if (request.getExamTaking() != null) {
                    long now = Timestamp.from(Instant.now()).getTime();
                    systemExamInfoDocumentSearchStream = systemExamInfoDocumentSearchStream
                            .filter(RdSystemExamInfoDocument$.BEGIN_TIME_OF_TS.le(now).and(RdSystemExamInfoDocument$.END_TIME_OF_TS.ge(now)));
                    String[] examTakingIds =
                            systemExamInfoDocumentSearchStream.collect(Collectors.toList())
                                    .stream().map(ii -> ii.getExamSiteRoomId())
                                    .collect(Collectors.toList())
                                    .toArray(it -> new String[it]);

                    if (request.getExamTaking()) {
                        if(examTakingIds.length==0) {
                            return Collections.EMPTY_LIST;
                        }

                        examSiteRoomDocumentStream
                                = examSiteRoomDocumentStream
                                .filter(RdExamSiteRoomDocument$.ID.in(examTakingIds));
                    } else if(examTakingIds.length>0){
                        examSiteRoomDocumentStream
                                = examSiteRoomDocumentStream
                                .filter(RdExamSiteRoomDocument$.ID.notEq(examTakingIds));
                    }
                }

                /**
                 * 是否关注
                 */
                if (request.getFollowed() != null) {
                    if (request.getFollowed()) {
                        examSiteRoomDocumentStream =
                                examSiteRoomDocumentStream.filter(
                                        RdEmployeeDocument$.FOLLOWED_DEVICE_NOS.in(userModel.getTencentUserId())
                                );
                    } else {
                        /**
                         * 关注人不在列表中
                         */
                        Set<String> noDeviceNos = new HashSet<>();
                        noDeviceNos.add(userModel.getTencentUserId());

                        examSiteRoomDocumentStream =
                                examSiteRoomDocumentStream.filter(
                                        RdEmployeeDocument$.FOLLOWED_DEVICE_NOS.containsNone(noDeviceNos)
                                );
                    }
                }

                /**
                 * 是否启用
                 */
                if (request.getStatus() != null) {
                    examSiteRoomDocumentStream =
                            examSiteRoomDocumentStream.filter(RdExamSiteRoomDocument$.STATUS.eq(request.getStatus().booleanValue()));
                }
            }
            case Keyword -> {
                if (!StringUtils.hasLength(request.getKeyword())) {
                    throw new BadTenantException("搜索关键字不允许为空");
                }
                examSiteRoomDocumentStream =
                        examSiteRoomDocumentStream.filter(TermUtil.getTermKeyword(request.getKeyword()));
            }
        }

        int pageIndex = request.getPageIndex();
        if (pageIndex < 1) {
            pageIndex = 1;
        }
        Collection<RdExamSiteRoomDocument> documents =
                examSiteRoomDocumentStream.skip((pageIndex - 1) * request.getPageSize())
                        .limit(request.getPageSize()).collect(Collectors.toList());

        return documents.stream()
                .map(ii -> ExamSiteRoomOverviewResponse.create(
                        request.getSearchModel(),
                        ii.getId(),
                        ii.getName(),
                        ii.getExamSiteName(), ii.getCompanyName(),
                        ii.getLonLat() != null ? ii.getLonLat().getY() : null,
                        ii.getLonLat() != null ? ii.getLonLat().getX() : null,
                        ii.isStatus()
                ))
                .collect(Collectors.toList());
    }

    @Override
    public ExamSiteRoomDetailResponse get(String id) {
        SearchStream<RdExamSiteRoomDocument> examSiteRoomDocumentStream =
                this.entityStream.of(RdExamSiteRoomDocument.class);
        examSiteRoomDocumentStream = examSiteRoomDocumentStream.filter(RdExamSiteRoomDocument$.ID.eq(id));
        RdExamSiteRoomDocument document =
                examSiteRoomDocumentStream.findFirst().orElse(null);

        if (document == null) {
            throw new NoFoundTenantException();
        }

        RdSystemExamInfoDocument examInfoDocument
                = extractCurrentExamInfo(document.getId());

        int countOfTeachers = 0;
        String examId = null;
        String examName = null;
        ExamType examType = null;
        LocalDateTime examBeginTime = null;
        LocalDateTime examEndTime = null;
        if (examInfoDocument != null) {
            countOfTeachers = examInfoDocument.getCountOfTeachers();
            examId = examInfoDocument.getId();
            examName = examInfoDocument.getName();
            examType = examInfoDocument.getExamType();
            examBeginTime = examInfoDocument.getBeginTime();
            examEndTime = examInfoDocument.getEndTime();
        }

        ExamSiteRoomDetailResponse response
                = ExamSiteRoomDetailResponse.create(
                document.getId(),
                document.getName(),
                document.getExamSiteId(),
                document.getExamSiteName(),
                document.getCompanyName(),
                document.getOrganizationId(),
                countOfTeachers,
                examType,
                examBeginTime, examEndTime,
                examId,
                examName,
                document.getCapacityOfPerson(),
                document.getScopeRange(),
                document.getLonLat().getY(),
                document.getLonLat().getX()
        );

        return response;
    }

    private RdSystemExamInfoDocument extractCurrentExamInfo(String examSiteRoomId) {
        SearchStream<RdSystemExamInfoDocument> systemExamInfoDocumentSearchStream =
                entityStream.of(RdSystemExamInfoDocument.class);
        systemExamInfoDocumentSearchStream =
                systemExamInfoDocumentSearchStream
                        .filter(RdSystemExamInfoDocument$.EXAM_SITE_ROOM_ID.eq(examSiteRoomId));

        Collection<RdSystemExamInfoDocument> systemExamInfoDocuments =
                systemExamInfoDocumentSearchStream.collect(Collectors.toList());
        long nowTs = Timestamp.from(Instant.now()).getTime();
        RdSystemExamInfoDocument selectedExamDocument
                = systemExamInfoDocuments.stream().filter(ii -> ii.getBeginTimeOfTs() <= nowTs && ii.getEndTimeOfTs() >= nowTs)
                .findFirst().orElse(systemExamInfoDocuments.stream().findFirst().orElse(null));

        return selectedExamDocument;
    }
}
