package com.bcxin.tenant.open.dubbo.reader.providers.impls;

import com.alibaba.fastjson2.util.DateUtils;
import com.bcxin.tenant.open.document.domains.documents.*;
import com.bcxin.tenant.open.document.domains.utils.KeywordTermUtils;
import com.bcxin.tenant.open.infrastructures.TenantContext;
import com.bcxin.tenant.open.infrastructures.TenantEmployeeContext;
import com.bcxin.tenant.open.infrastructures.exceptions.UnAuthorizedTenantException;
import com.bcxin.tenant.open.infrastructures.utils.BusinessUtil;
import com.bcxin.tenant.open.jdks.ProjectReaderRpcProvider;
import com.bcxin.tenant.open.jdks.requests.ProjectSearchRequest;
import com.bcxin.tenant.open.jdks.responses.ProjectOverviewResponse;
import com.redis.om.spring.search.stream.EntityStream;
import com.redis.om.spring.search.stream.SearchStream;
import org.apache.dubbo.config.annotation.DubboService;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.sql.Timestamp;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.time.Instant;
import java.util.*;
import java.util.stream.Collectors;

@DubboService
public class ProjectReaderRpcProviderImpl implements ProjectReaderRpcProvider {
    private final EntityStream entityStream;

    public ProjectReaderRpcProviderImpl(EntityStream entityStream) {
        this.entityStream = entityStream;
    }

    @Override
    public Collection<ProjectOverviewResponse> search(ProjectSearchRequest request) {
        int pageIndex = request.getPageIndex();
        if (pageIndex < 1) {
            pageIndex = 1;
        }
        TenantEmployeeContext.TenantUserModel userModel =
                TenantContext.getInstance().getUserContext().get();
        if (userModel == null) {
            throw new UnAuthorizedTenantException();
        }

        /**
         * 根据监管归属进行过滤, 暂时不支持企业级的调度台过滤
         */
        SearchStream<RdProjectDocument> projectDocumentSearchStream =
                this.entityStream.of(RdProjectDocument.class);

        projectDocumentSearchStream = filterByKeyword(projectDocumentSearchStream, request.getKeyword());
        if (userModel.isSupervise()) {
            String[] superviseDepartIds =
                    userModel.getMatchedSuperviseDepartIds(request.getSuperviseDepartIds())
                            .toArray(ii -> new String[ii]);

            if (superviseDepartIds.length == 0) {
                return Collections.EMPTY_LIST;
            }

            /**
             * 根据监管归属进行过滤, 暂时不支持企业级的调度台过滤
             */
            projectDocumentSearchStream = projectDocumentSearchStream.filter(
                    RdSecurityStationDocument$.SCOPE_PERMISSIONS
                            .in(superviseDepartIds)
            );
        } else {
            /**
             * 本单位的项目信息
             */
            Set<String> scopes = new HashSet<>();
            scopes.add(userModel.getOrganizationId());
            /**
             * 承办商关联的数据信息
             */
            if (BusinessUtil.isEventOrganizer(userModel.getOrgInstitutional())) {
                scopes.addAll(Arrays.stream(userModel.getNoSuperviseScopePermissions()).toList());
            }

            projectDocumentSearchStream = projectDocumentSearchStream.filter(
                    RdSecurityStationDocument$.SCOPE_PERMISSIONS
                            .in(scopes.toArray(ix -> new String[ix]))
            );
        }

        Collection<RdProjectDocument> documents =
                projectDocumentSearchStream
                        .skip((pageIndex - 1) * request.getPageSize())
                        .limit(request.getPageSize())
                        .collect(Collectors.toList());

        return documents.stream().map(
                ii -> {
                    ProjectOverviewResponse rp = ProjectOverviewResponse.create(ii.getId(), ii.getName(), ii.getBeginDate(), ii.getEndDate());
                    rp.setScopePermissions(ii.getScopePermissions());

                    return rp;
                }
        ).collect(Collectors.toSet());
    }

    @Override
    public Collection<String> getIdsByOrganIds(Collection<String> organIds) {
        if (CollectionUtils.isEmpty(organIds)) {
            return Collections.EMPTY_LIST;
        }

        SearchStream<RdProjectDocument> projectDocumentSearchStream =
                this.entityStream.of(RdProjectDocument.class);

        projectDocumentSearchStream = projectDocumentSearchStream.filter(
                RdSecurityStationDocument$.SCOPE_PERMISSIONS
                        .in(organIds.toArray(ix -> new String[ix]))
        );
        Collection<RdProjectDocument> documents =
                projectDocumentSearchStream
                        .collect(Collectors.toList());

        return documents.stream().map(ii -> ii.getId()).collect(Collectors.toSet());
    }

    private static SearchStream<RdProjectDocument> filterByKeyword(
            SearchStream<RdProjectDocument> searchStream,
            String input) {
        if (StringUtils.hasLength(input)) {
            String searchKeyword = input.trim();
            String expression = KeywordTermUtils.getFilterExpression(searchKeyword);
            searchStream = searchStream.filter(expression);
        }

        return searchStream;
    }
}
